package com.nisum.example.service;

import com.nisum.example.model.Reservation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.mongodb.core.ReactiveMongoOperations;
import org.springframework.data.mongodb.core.query.Criteria;
import org.springframework.data.mongodb.core.query.Query;
import org.springframework.data.mongodb.core.query.Update;
import org.springframework.stereotype.Service;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;

import java.util.List;

/**
 * Responsible to perform get, post, delete and update operations
 */

@Service
public class ReservationServiceImpl implements ReservationService {

    private final ReactiveMongoOperations reactiveMongoOperations;

    @Autowired
    public ReservationServiceImpl(ReactiveMongoOperations reactiveMongoOperations) {
        this.reactiveMongoOperations = reactiveMongoOperations;
    }

    /**
     * perform fetch operation based on id
     * @param Id
     * @return One entity -Mono<T>
     */
    @Override
    public Mono<Reservation> getReservation(String Id) {
        return reactiveMongoOperations.findById(Id,Reservation.class);
    }

    @Override
    public Flux<Reservation> getAllReservations() {
        return reactiveMongoOperations.findAll(Reservation.class);
    }

    /**
     * perform save operation of one entity
     * @param reservationMono
     * @return same entity as type Mon<T>
     */
    @Override
    public Mono<Reservation> createReservation(Mono<Reservation> reservationMono) {
        return reactiveMongoOperations.save(reservationMono);
    }

    /**
     * perform save operation of one entities as List collection
     * @param reservationList
     * @return same entities as type Flux<T>
     */

    @Override
    public Flux<Reservation> createReservations(List<Reservation> reservationList) {
        return reactiveMongoOperations.insert(reservationList,List.class);
    }

    /**
     * perform update operation by taking id and entity as an input
     * @param id
     * @param reservationMono
     * @return same entity of type Mono<T>
     */
    @Override
    public Mono<Reservation> updateReservation(String id, Mono<Reservation> reservationMono) {
      //if data already present do update
     // return reactiveMongoOperations.save(reservationMono);

        //update just price
        return reservationMono.flatMap(reservation ->
            reactiveMongoOperations.findAndModify(Query.query(Criteria.where("id").is(id)),
                    Update.update("price",reservation.getPrice()),Reservation.class)
                   .flatMap(result -> {
                       result.setPrice(reservation.getPrice());
                       return Mono.just(result);
                   })
        );
    }

    /**
     * perform delete operation by taking id as input
     * @param id
     * @return boolean only if record is deleted successfully
     */
    @Override
    public Mono<Boolean> deleteReservation(String id) {
        return reactiveMongoOperations.remove(Query.query(Criteria.where("id").is(id)),Reservation.class)
                .flatMap(deleteResult -> Mono.just(deleteResult.wasAcknowledged()));
    }
}
